@@ -1,10 +1,10 @@
 from dataclasses import dataclass
 from typing import Any, Dict, Optional, Tuple
 
-from pytorch_ie.core.document import Annotation
+from pytorch_ie.core.document import Annotation, resolve_annotation
 
 
-def _validate_single_label(self):
+def _post_init_single_label(self):
     if not isinstance(self.label, str):
         raise ValueError("label must be a single string.")
 
@@ -12,7 +12,7 @@ def _validate_single_label(self):
         raise ValueError("score must be a single float.")
 
 
-def _validate_multi_label(self):
+def _post_init_multi_label(self):
     if self.score is None:
         score = tuple([1.0] * len(self.label))
         object.__setattr__(self, "score", score)
@@ -29,22 +29,27 @@ def _validate_multi_label(self):
         )
 
 
+def _post_init_multi_span(self):
+    if isinstance(self.slices, list):
+        object.__setattr__(self, "slices", tuple(tuple(s) for s in self.slices))
+
+
 @dataclass(eq=True, frozen=True)
 class Label(Annotation):
     label: str
     score: float = 1.0
 
     def __post_init__(self) -> None:
-        _validate_single_label(self)
+        _post_init_single_label(self)
 
 
 @dataclass(eq=True, frozen=True)
 class MultiLabel(Annotation):
-    label: Tuple[str]
-    score: Optional[Tuple[float]] = None
+    label: Tuple[str, ...]
+    score: Optional[Tuple[float, ...]] = None
 
     def __post_init__(self) -> None:
-        _validate_multi_label(self)
+        _post_init_multi_label(self)
 
 
 @dataclass(eq=True, frozen=True)
@@ -64,42 +69,38 @@ class LabeledSpan(Span):
     score: float = 1.0
 
     def __post_init__(self) -> None:
-        _validate_single_label(self)
+        _post_init_single_label(self)
 
 
 @dataclass(eq=True, frozen=True)
 class MultiLabeledSpan(Span):
-    label: Tuple[str]
-    score: Optional[Tuple[float]] = None
+    label: Tuple[str, ...]
+    score: Optional[Tuple[float, ...]] = None
 
     def __post_init__(self) -> None:
-        _validate_multi_label(self)
+        _post_init_multi_label(self)
 
 
 @dataclass(eq=True, frozen=True)
 class LabeledMultiSpan(Annotation):
-    slices: Tuple[Tuple[int, int]]
+    slices: Tuple[Tuple[int, int], ...]
     label: str
     score: float = 1.0
 
     def __post_init__(self) -> None:
-        if isinstance(self.label, list):
-            object.__setattr__(self, "slices", tuple(self.slices))
-
-        _validate_single_label(self)
+        _post_init_multi_span(self)
+        _post_init_single_label(self)
 
 
 @dataclass(eq=True, frozen=True)
 class MultiLabeledMultiSpan(Annotation):
-    slices: Tuple[Tuple[int, int]]
-    label: Tuple[str]
-    score: Optional[Tuple[float]] = None
+    slices: Tuple[Tuple[int, int], ...]
+    label: Tuple[str, ...]
+    score: Optional[Tuple[float, ...]] = None
 
     def __post_init__(self) -> None:
-        if isinstance(self.label, list):
-            object.__setattr__(self, "slices", tuple(self.slices))
-
-        _validate_multi_label(self)
+        _post_init_multi_span(self)
+        _post_init_multi_label(self)
 
 
 @dataclass(eq=True, frozen=True)
@@ -110,81 +111,50 @@ class BinaryRelation(Annotation):
     score: float = 1.0
 
     def __post_init__(self) -> None:
-        _validate_single_label(self)
+        _post_init_single_label(self)
 
     def asdict(self) -> Dict[str, Any]:
-        dct = super().asdict()
-        dct["head"] = hash(self.head)
-        dct["tail"] = hash(self.tail)
+        dct = self._asdict(overrides={"head": self.head._id, "tail": self.tail._id})
         return dct
 
     @classmethod
     def fromdict(
         cls,
         dct: Dict[str, Any],
-        annotation_store: Optional[Dict[int, Tuple[str, "Annotation"]]] = None,
+        annotation_store: Optional[Dict[int, Annotation]] = None,
     ):
+        # copy to not modify the input
         tmp_dct = dict(dct)
-        tmp_dct.pop("_id", None)
-
-        head = tmp_dct["head"]
-        tail = tmp_dct["tail"]
-
-        if isinstance(head, int):
-            if annotation_store is None:
-                raise ValueError("Unable to resolve head reference without annotation_store.")
-
-            tmp_dct["head"] = annotation_store[head][1]
-
-        if isinstance(tail, int):
-            if annotation_store is None:
-                raise ValueError("Unable to resolve tail reference without annotation_store.")
-
-            tmp_dct["tail"] = annotation_store[tail][1]
-
-        return cls(**tmp_dct)
+        tmp_dct["head"] = resolve_annotation(tmp_dct["head"], store=annotation_store)
+        tmp_dct["tail"] = resolve_annotation(tmp_dct["tail"], store=annotation_store)
+        return super().fromdict(tmp_dct, annotation_store)
 
 
 @dataclass(eq=True, frozen=True)
 class MultiLabeledBinaryRelation(Annotation):
     head: Span
     tail: Span
-    label: Tuple[str]
-    score: Optional[Tuple[float]] = None
+    label: Tuple[str, ...]
+    score: Optional[Tuple[float, ...]] = None
 
     def __post_init__(self) -> None:
-        _validate_multi_label(self)
+        _post_init_multi_label(self)
 
     def asdict(self) -> Dict[str, Any]:
-        dct = super().asdict()
-
         # replace object references with object hashes
-        dct["head"] = hash(self.head)
-        dct["tail"] = hash(self.tail)
+        dct = self._asdict(overrides={"head": self.head._id, "tail": self.tail._id})
         return dct
 
     @classmethod
     def fromdict(
         cls,
         dct: Dict[str, Any],
-        annotation_store: Optional[Dict[int, Tuple[str, "Annotation"]]] = None,
+        annotation_store: Optional[Dict[int, "Annotation"]] = None,
     ):
         tmp_dct = dict(dct)
         tmp_dct.pop("_id", None)
 
-        head = tmp_dct["head"]
-        tail = tmp_dct["tail"]
-
-        if isinstance(head, int):
-            if annotation_store is None:
-                raise ValueError("Unable to resolve head reference without annotation_store.")
-
-            tmp_dct["head"] = annotation_store[head][1]
-
-        if isinstance(tail, int):
-            if annotation_store is None:
-                raise ValueError("Unable to resolve tail reference without annotation_store.")
-
-            tmp_dct["tail"] = annotation_store[tail][1]
+        tmp_dct["head"] = resolve_annotation(tmp_dct["head"], store=annotation_store)
+        tmp_dct["tail"] = resolve_annotation(tmp_dct["tail"], store=annotation_store)
 
         return cls(**tmp_dct)
