@@ -27,6 +27,14 @@ class PyTorchIEBaseModelHubMixin:
 
     config_name = CONFIG_NAME
 
+    def __init__(self, *args, is_from_pretrained: bool = False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._is_from_pretrained = is_from_pretrained
+
+    @property
+    def is_from_pretrained(self):
+        return self._is_from_pretrained
+
     def save_pretrained(
         self,
         save_directory: str,
@@ -165,6 +173,9 @@ class PyTorchIEBaseModelHubMixin:
 
         config.update(model_kwargs)
 
+        # the entry may be already in the config, so we overwrite it
+        config["is_from_pretrained"] = True
+
         return cls._from_pretrained(
             model_id,
             revision,
@@ -317,6 +328,7 @@ class PyTorchIEModelHubMixin(PyTorchIEBaseModelHubMixin):
             >>> # Downloading weights from hf-hub & model will be initialized from those weights
             >>> model = MyModel.from_pretrained("username/mymodel@main")
         """
+        super().__init__(*args, **kwargs)
 
     def _save_pretrained(self, save_directory):
         """
@@ -397,7 +409,7 @@ class PyTorchIETaskmoduleModelHubMixin(PyTorchIEBaseModelHubMixin, Hyperparamete
             >>> # Downloading weights from hf-hub & model will be initialized from those weights
             >>> model = MyModel.from_pretrained("username/mymodel@main")
         """
-        super().__init__()
+        super().__init__(*args, **kwargs)
 
     def _config(self) -> Dict[str, Any]:
         config = dict(self.hparams)
